import os, tokenize
from Core import Command
from Tools import pygettext

class GenerateL10n(Command):

    command_name = 'generate_l10n'

    description = "generate localizations"

    user_options = [
        ('force', 'f',
         'force locale generatation (overwrite existing files)'),
        ]

    boolean_options = ['force']

    def initialize_options(self):
        self.outfiles = []
        self.l10n = None
        self.force = 0

    def finalize_options(self):
        self.set_undefined_options('generate', ('force', 'force'))
        self.l10n = self.distribution.l10n

    def get_outputs(self):
        return self.outfiles

    def run(self):
        if not self.l10n:
            return

        class Options:
            # constants
            GNU = 1
            SOLARIS = 2
            # defaults
            extractall = 0 # FIXME: currently this option has no effect at all.
            # replace non-ASCII characters with octal escape sequences
            escape = 0
            # keywords to mark strings as candidates for translation
            keywords = ['_']
            outpath = ''
            # where to store gathered messages (language + .po)
            outfile = ''
            writelocations = 1
            locationstyle = GNU
            verbose = 0
            width = 78
            excludefilename = ''
            # a list of strings to exclude from gathering
            toexclude = []

        for loc in self.l10n:
            # Each localization group shares Options
            options = Options()
            options.outfile = loc.language + '.po'
            eater = pygettext.TokenEater(options)
            for source in loc.sources:
                self.announce("scanning for messages in '%s'" % source)
                fp = open(source)
                pygettext.make_escapes(options.escape)
                eater.set_filename(source)
                tokenize.tokenize(fp.readline, eater)
                fp.close()

            po_dir = self.get_package_dir(loc.package)
            po_filename = os.path.join(po_dir, options.outfile)
            self.announce("generating message file '%s'" % po_filename)
            # Binary mode is needed when generating on non-LF terminating
            # platforms (i.e. Windows, Mac).  Python can always read LF
            # terminated files.
            fp = open(po_filename, 'wb')
            eater.write(fp)
            fp.close()
        return