import os, types
from distutils import core
from distutils import fancy_getopt
from distutils.errors import *
from distutils.util import convert_path
from distutils.version import StrictVersion
from pprint import pprint
import Dist

core.USAGE = """\
Usage:
  %(script)s [global_opts] cmd1 [cmd1_opts] [cmd2 [cmd2_opts] ...]
  %(script)s --help [cmd1 cmd2 ...]
  %(script)s cmd --help
  %(script)s --help-commands
  %(script)s --help-packages
"""

# the wrap_text function will collapse 2+ newlines into 1+ newlines
INSTALL_SUCCESS = """\
%(name)s version %(version)s has been successfully installed!


Data directory: %(install_data)s

The data directory includes documentation, demos and test suites.
The documentation is in text, html and xml formats.


Library code is installed at:

%(install_lib)s%(package_root)s.

You probably want to add to your executable path:

%(install_scripts)s.

Documentation is installed at:

%(install_docs)s.

Tests are installed at

%(install_tests)s.

Execute tests by changing to this directory and running:

"python test.py Path/To/Test/Module"
"""


# Find the package database
def GetPackageDb():
    import sys
    old_ft = sys.modules.get('Ft')
    if old_ft:
        del sys.modules['Ft']
    local = sys.path[0]
    del sys.path[0]
    try:
        from Ft import PACKAGE_DATABASE
        del sys.modules['Ft']
    except ImportError:
        sys.path.insert(0, local)
        return None

    sys.path.insert(0, local)
    # Put the original Ft back
    if old_ft:
        sys.modules['Ft'] = old_ft
    return PACKAGE_DATABASE

_PACKAGE_DB = GetPackageDb()

def find_package_info(manager, dirname, names):
    if manager.package_file in names:
        filename = manager.package_file
    elif manager.package_file + '.py' in names:
        filename = manager.package_file + '.py'
    else:
        # No package defined in this directory
        return

    # We found a package, remove the rest of the files
    # This prevents further walking of this tree
    names[:] = []

    package = {}
    filename = os.path.join(dirname, filename)
    execfile(filename, {}, package)

    name = package.get('name')
    if not name:
        print "Unnamed package from '%s', skipped" % filename
    elif manager.package_info.has_key(name):
        print "Duplicate package for '%s', ignoring '%s'" % (name, filename)
    else:
        manager.package_info[name] = package
    return

class PackageManager(Dist.Dist):

    global_options = Dist.Dist.global_options + [
        ('package=', 'p', "limit command(s) to a given package"),
        ]
    
    display_options = [
        ('help-packages', None, 'list all available packages'),
        ] + Dist.Dist.display_options

    display_option_names = ['help_packages'] + Dist.Dist.display_option_names

    def __init__(self, attrs):
        self.package = None
        self.package_file = None
        self.package_root = None
        Dist.Dist.__init__(self, attrs)
        return

    def finalize_options(self):
        Dist.Dist.finalize_options(self)
        
        # Search out available packages
        self.package_info = {}
        root_dir = self.package_root or '.'
        os.path.walk(root_dir, find_package_info, self)
        return

    def handle_display_options(self, option_order):
        from distutils.core import gen_usage
        if self.help_packages:
            self.print_packages()
            print
            print gen_usage(self.script_name)
            return 1
        return Dist.Dist.handle_display_options(self, option_order)

    def print_packages(self):
        packages = self.package_info.keys()
        packages.sort()

        max_length = 0
        for pkg in packages:
            if len(pkg) > max_length:
                max_length = len(pkg)

        print "Available packages:"
        for pkg in packages:
            try:
                description = self.package_info[pkg]['description']
            except KeyError:
                description = '(no description available)'

            print "  %-*s  %s" % (max_length, pkg, description)
        return

    def run_commands(self):
        if self.package:
            if not self.package_info.has_key(self.package):
                raise DistutilsArgError, \
                      "package '%s' is unknown, use --help-packages " \
                      "to get a complete listing" % self.package
            packages = self.resolve_dependencies(self.package)
        else:
            packages = self.package_info.keys()

        # Build a composite of requested package and dependencies
        for name in self.sort_dependencies(packages):
            self.merge_package(self.package_info[name])

        Dist.Dist.run_commands(self)

        if self.have_run.get('install') and not self.developer_mode:
            install = self.get_command_obj('install')
            subst = {}
            subst.update(vars(self))
            subst.update(vars(self.metadata))
            subst.update(vars(install))

            message = INSTALL_SUCCESS % subst
            print '-'*78
            print wrap_text(message, 78)
        return

    def merge_package(self, package_info, overwrite=0):
        # Any attribute that is not already defined is invalid!
        metadict = self.metadata.__dict__
        dict = self.__dict__
        for key, val in package_info.items():
            if metadict.has_key(key):
                if overwrite:
                    metadict[key] = val
            elif dict.has_key(key):
                if type(val) is types.TupleType:
                    val = list(val)
                if overwrite or dict[key] is None:
                    dict[key] = val
                elif type(dict[key]) is type(val) is types.ListType:
                    # Merge two lists
                    dict[key].extend(val)
                else:
                    raise DistutilsSetupError, \
                          "incompatiable types for option '%s'" % key
            elif key not in ['dependencies']:
                raise DistutilsSetupError, \
                      "invalid distribution option '%s'" % key
        return

    def sort_dependencies(self, packages, sorted=None):
        if sorted is None:
            # First time through
            # Remove any duplicates
            temp = []
            for name in packages:
                if name not in temp:
                    temp.append(name)
            packages = temp
            sorted = []

        last_package = None
        while packages:
            package = packages[0]
            if last_package == package:
                # FIXME - when this becomes an issue
                # currently we don't have any, so why worry?
                raise Exception("unresolved/circular dependency: %s" % packages)
            # Not all packages will be in the package_info mapping
            #  (might already be installed)
            info = self.package_info.get(package)
            if not info:
                packages.remove(package)
                continue

            dependencies = info.get('dependencies')
            if dependencies:
                for name in sorted:
                    dels = filter(lambda (d, v), n=name: d == n, dependencies)
                    for dep in dels:
                        dependencies.remove(dep)
                if dependencies:
                    # Move it to the end
                    packages.append(package)
                    del packages[0]
            if not dependencies:
                sorted.append(package)
                del packages[0]
        return sorted

    def resolve_dependencies(self, package):
        packages = [package]
        pkg_info = self.package_info[package]
        for (name, version) in pkg_info.get('dependencies', []):
            required_version = StrictVersion(version)
            if self.package_info.has_key(name):
                dep_pkg = self.package_info[name]
                dep_version = StrictVersion(dep_pkg['version'])
                if dep_version != required_version:
                    msg = 'version conflict for %s, requires %s, installing %s' % (
                        name, required_version, dep_version)
                    raise DistutilsFileError('failed dependencies: ' + msg)
                packages.extend(self.resolve_dependencies(name))
            elif not self.installed_version(name, strict_version):
                raise DistuilsFileError('missing dependency: %s-%s' % (name, version))
        return packages

    def installed_version(self, name, version):
        global _PACKAGE_DB

        if _PACKAGE_DB is None:
            # Nothing, not even the database, is installed yet
            return 0

        if type(_PACKAGE_DB) is not type({}):
            # First time through, load the database
            try:
                dict = eval(open(_PACKAGE_DB).read())
            except IOError, error:
                raise DistutilsFileError, \
                      'Cannot read package database: ' + str(error)
            except:
                # Error during eval()
                raise DistutilsFileError, \
                      'Invalid package database: %s' % _PACKAGE_DB

            if type(dict) is not type({}):
                raise DistutilsFileError, \
                      'Invalid package database: %s' % _PACKAGE_DB
            _PACKAGE_DB = dict

        if _PACKAGE_DB.has_key(name):
            # Check the package information 
            package_info = _PACKAGE_DB[name]
            if package_info.has_key('version'):
                return StrictVersion(package_info['version']) == version
            else:
                raise DistutilsFileError, \
                      "Invalid package information for '%s'" % name
        # Package is not installed
        return 0

def wrap_text (text, width):
    if text is None: return ''
    if len(text) <= width: return text

    chunks = text.split('\n\n')
    result = []
    for chunk in chunks:
        if chunk.startswith('\n'):
            # An odd number of newlines, the empty string will become
            # a newline when this list is joined.
            result.append('')
            chunk = chunk[1:]
        lines = fancy_getopt.wrap_text(chunk, width)
        result.append('\n'.join(lines))

    return '\n'.join(result)


