########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Add/Statement.py,v 1.8 2003/03/20 03:50:18 mbrown Exp $
"""
Implementation of '4ss add statement' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Adds a single statement to the RDF model."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes
from Ft.Rdf import Statement, OBJECT_TYPE_LITERAL, OBJECT_TYPE_UNKNOWN

OTYPES = {'literal': OBJECT_TYPE_LITERAL,
          'resource': OBJECT_TYPE_UNKNOWN,
          }

def Run(options,args):

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.add.statement')
        try:
            model = repo.getModel()
            scope = options.get('scope', '')
            if scope and options.get('create'):
                #Create an empty RDF doc
                doc = repo.createDocument(
                    scope, '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"/>',
                    imt=options.get('imt', 'text/xml'),
                    forcedType=ResourceTypes.ResourceType.RDF_DOCUMENT,
                )
            model.add([Statement.Statement(
                args['subject'],
                args['predicate'],
                args['object'],
                args.get('uri', ''),
                scope,
                objectType = OTYPES.get(options.get('objtype'), OBJECT_TYPE_LITERAL)
                )])
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('statement',
                          """Add a statement to the the RDF model.""",
                          'ftss:///repo/path/to/doc.xml http://purl.org/dc/elements/1.1/title "Interesting Data"',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('subject',
                                                                  "The subject of the new statement.",
                                                                  str),
                                       Arguments.RequiredArgument('predicate',
                                                                  "The predicate of the new statement.",
                                                                  str),
                                       Arguments.RequiredArgument('object',
                                                                  "The object of the new statement.",
                                                                  str),
                                       Arguments.OptionalArgument('uri',
                                                                  "The URI of the new statement.",
                                                                  str),
                                       ],

                          options = Options.Options([Options.Option(None, 'scope=', "The scope of the statement"),
                                                     Options.Option(None, 'create', "Create the virtual RDF document representing the scope, if it doesn't already exist"),
                                                     Options.Option(None, 'objtype=', "The type of the object.  Usually 'resource' or 'literal'.  'literal' by default."),
                                                     ]),
                          fileName = __file__,
                          )
    return cmd

