########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Create/Document.py,v 1.13 2003/03/20 03:50:18 mbrown Exp $
"""
Implementation of '4ss create document' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Creates a new XML Document in the 4Suite repository.
This command is used to store XML-based data in the repository.
If you are storing non-XML data, see the 'create rawfile' command."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Lib import Uri
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

forcedTypeMapping = {'xml':ResourceTypes.ResourceType.XML_DOCUMENT,
                     'xslt':ResourceTypes.ResourceType.XSLT_DOCUMENT,
                     'schematron':ResourceTypes.ResourceType.SCHEMATRON_DOCUMENT,
                     'rdf':ResourceTypes.ResourceType.RDF_DOCUMENT,
                     'container':ResourceTypes.ResourceType.CONTAINER,
                     'user':ResourceTypes.ResourceType.USER,
                     'xpathdocdef':ResourceTypes.ResourceType.XPATH_DOCUMENT_DEFINITION,
                     'xsltdocdef':ResourceTypes.ResourceType.XSLT_DOCUMENT_DEFINITION,
                     'group':ResourceTypes.ResourceType.GROUP,
                     'alias':ResourceTypes.ResourceType.ALIAS,
                     }


def Run(options,args):

    if args['sourceUri'] == '-':
        sourceStream = sys.stdin
    else:
        sourceStream = Uri.BASIC_RESOLVER.resolve(args['sourceUri'])

    text = sourceStream.read()

    name = args['name']
    aliases = args.get('alias',[])

    try:

        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.create.document')

        docDef=options.get('docdef', None)

        try:
            doc = repo.createDocument(name,
                                      text,
                                      imt=options.get('imt', 'text/xml'),
                                      docDef=docDef,
                                      forcedType=forcedTypeMapping.get(options.get('type'))
                                      )
            for alias in aliases:
                doc.addAlias(alias)
            commit = 1

        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')

    if args['sourceUri'] != '-':
        sourceStream.close()



def Register():
    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('document',
                          'Creates a new XML Document in the 4Suite repository.',
                          '/repo/path/to/dummy.xml dummy.xml',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('name',
                                                                  "The name (repo path) of the new document",
                                                                  str),
                                       Arguments.RequiredArgument('sourceUri',
                                                                  "The URI to read the document's content from, or '-' for stdin",
                                                                  str),
                                       Arguments.ZeroOrMoreArgument('alias',
                                                                    "Additional aliases to assign to the new document.",
                                                                    str),
                                       ],
                          options = Options.Options([Options.Option('i',
                                                                    'imt=IMT',
                                                                    'The document\'s Internet Media Type. Default: text/xml.'),
                                                     Options.Option('D',
                                                                    'docdef=docdef',
                                                                    "The name of the document definition to use."),
                                                     Options.TypedOption('t',
                                                                         'type=',
                                                                         'The 4SS document type. If none specified, then the' \
                                                                         ' type is inferred from the document\'s content.',
                                                                         [('xml',"generic XML Document (this is the default)"),
                                                                          ('xslt', "XSLT Document"),
                                                                          ('schematron', "Schematron Document"),
                                                                          ('rdf',"RDF Document"),
                                                                          ('container',"Container"),
                                                                          ('user',"User"),
                                                                          ('xpathdocdef',"XPath Document Definition"),
                                                                          ('xsltdocdef',"XSLT Document Definition"),
                                                                          ('group',"Group"),
                                                                          ('alias',"Alias")
                                                                          ]),
                                                     ]),

                          fileName = __file__,
                          )
    return cmd
