########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Rdf/Remove.py,v 1.10 2003/03/20 03:50:19 mbrown Exp $
"""
Implementation of '4ss rdf remove' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Remove statements from the RDF model.
A single statement can be given via command-line arguments, or multiple statements can be given in an RDF XML file that will be deserialized."""

import sys
from Ft.Lib import Uri
from Ft.Rdf import Model
from Ft.Rdf.Statement import Statement
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil


def Run(options, args):
    flags = {}
    if options.get('subject-regex'):
        flags['subjectFlags'] = Model.REGEX
    if options.get('predicate-regex'):
        flags['predicateFlags'] = Model.REGEX
    if options.get('object-regex'):
        flags['objectFlags'] = Model.REGEX
    subject = options.get('subject')
    predicate = options.get('predicate')
    object = options.get('object')
    try:
        commit = 0
        repo = CommandUtil.GetRepository(options, '4ss.rdf.remove')
        try:
            scope = args.get('scope')
            if options.get('document'):
                doc = repo.fetchResource(scope)
                stream = Uri.BASIC_RESOLVER.resolve(options['document'])
                src = stream.read()
                doc.deserializeAndRemove(src)
            else:
                model = repo.getModel()
                #FIXME: make requirment that they must be superuser to
                #Remove a statement without specified scope
                apply(model.removePattern,
                      (subject, predicate, object, None, scope),
                      flags)
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('remove',
                          'Remove RDF statements from a given scope',
                          '--document=spam.rdf /my/scope/path',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.OptionalArgument('scope',
                                                                  "The scope from which the statements are to be removed",
                                                                  str)],
                          options = Options.Options([
                              Options.Option('d', 'document=', 'Remove statements deserialized from the given document'),
                              Options.Option(None, 'subject=', 'The subject of statements selected to be deleted'),
                              Options.Option(None, 'predicate=', 'The predicate of statements selected to be deleted'),
                              Options.Option(None, 'object=', 'The object of statements selected to be deleted'),
                              Options.Option(None, 'subject-regex', 'The subject string is a regular expression to match'),
                              Options.Option(None, 'predicate-regex', 'The predicate string is a regular expression to match'),
                              Options.Option(None, 'object-regex', 'The object string is a regular expression to match'),
                                      ]),
                          fileName = __file__,
                          )
    return cmd

