<?xml version="1.0"?>
<!--

 File Name:            api.xslt

 Documentation:        http://docs.4suite.com/stylesheets/api.xslt.html

 Stylesheet to generate 4Suite API docs
 WWW: http://4suite.com/4XSLT        e-mail: support@4suite.com

 Copyright (c) 1999-2000 Fourthought Inc, USA.   All Rights Reserved.
 See  http://4suite.com/COPYRIGHT  for license and copyright information
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:tech="http://namespaces.fourthought.com/technical"
  xmlns:doc="http://docbook.org/docbook/xml/4.0/namespace"
  xmlns:ft="http://xmlns.4suite.org/ext"
  xmlns:dc= "http://purl.org/dc/elements/1.1/"
  version="1.0"
>

  <xsl:import href='docbook_html1.xslt'/>

  <xsl:import href='api-typedef.xslt'/>
  <xsl:import href='api-exception.xslt'/>
  <xsl:import href='api-structure.xslt'/>
  <xsl:import href='api-commandline.xslt'/>
  <xsl:import href='api-class.xslt'/>

  <xsl:output method="html" encoding="ISO-8859-1"/>

  <xsl:key name="Names" match="tech:*" use='tech:Name'/>

  <xsl:variable name='doc-css-style'>
    <xsl:comment>
      BODY        {background: white}
      A:link      {font-weight: 600;
                   font-family: sans-serif;
                   color: blue;
                   text-decoration: none}
      A:visited   {font-weight: 600;
                   font-family: sans-serif;
                   color: #800080;
                   text-decoration: none}
      A:active    {font-weight: 600;
                   font-family: sans-serif;
                   color: #FF0000;
                   text-decoration: none}
      A:hover     {font-weight: 600;
                   font-family: sans-serif;
                   color: #FF0000;
                   text-decoration: none}
    </xsl:comment>
  </xsl:variable>

  <xsl:param name="FULL_DOCUMENT" select="1"/>

  <xsl:variable name="doc-title" select="/*/@title"/>

  <xsl:template match="/">
    <xsl:choose>
    <xsl:when test="$FULL_DOCUMENT = 1">
      <HTML>
      <HEAD>
        <TITLE><xsl:value-of select='$doc-title'/></TITLE>
        <STYLE TYPE='text/css'>
          <xsl:copy-of select='$doc-css-style'/>
        </STYLE>
      </HEAD>
      <BODY>
        <H1><xsl:value-of select='$doc-title'/></H1>
        <BR/>
        <xsl:apply-templates/>
      </BODY>
      </HTML>
    </xsl:when>
    <xsl:otherwise>
      <H1><xsl:value-of select='$doc-title'/></H1>
      <BR/>
      <xsl:apply-templates/>
    </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template match="tech:Module">
    <H2>Module <xsl:value-of select='tech:Name'/></H2>
    <xsl:if test='tech:Desc'>  
      <P><xsl:value-of select='tech:Desc'/></P>
    </xsl:if>
    <p>Module Summary</p>
    <ul>
    <xsl:if test='tech:CommandLine'>
      <li><a href="#COMMAND_LINE">Command Line information</a></li>
    </xsl:if>  
    <xsl:if test='tech:Enum | tech:TypeDef | tech:Constant | tech:Union'>
      <li><a href="#TYPE_DEF">Types, Enums, Unions and constants</a></li>
    </xsl:if>  
    <xsl:if test='tech:Exception'>
      <li><a href="#EXCEPTION">Exceptions</a></li>
    </xsl:if>  
    <xsl:if test='tech:Structure'>
      <li><a href="#STRUCTURE">Structures</a></li>
    </xsl:if>  
    <xsl:if test='tech:Class'>
      <li><a href="#CLASS">Classes</a></li>
    </xsl:if>  
    <xsl:if test='tech:Function'>
      <li><a href="#FUNCTION">Global Functions</a></li>
    </xsl:if>
    </ul>
    <A name="COMMAND_LINE"/>
    <xsl:if test='tech:CommandLine'>
       <H3>Command Line</H3>
       <xsl:apply-templates select='tech:CommandLine'/>
    </xsl:if>
    <A name="TYPE_DEF"/>
    <xsl:if test='tech:Enum | tech:TypeDef | tech:Constant | tech:Union'>
       <H3>Type Defs, Enums, Unions and Constants</H3>
       <xsl:apply-templates select='tech:Enum | tech:TypeDef | tech:Constant | tech:Union'/>
    </xsl:if>
    <A name="EXCEPTION"/>
    <xsl:if test='tech:Exception'>
       <H3>Exceptions</H3>
         <xsl:apply-templates select='tech:Exception'/>
    </xsl:if>
    <A name="STRUCTURE"/>
    <xsl:if test='tech:Structure'>
       <H3>Structures</H3>
         <xsl:apply-templates select='tech:Structure'/>
    </xsl:if>
    <A name="CLASS"/>
    <xsl:if test='tech:Class'>
       <H3>Classes</H3>
       <xsl:call-template name="class-summary"/>
       <xsl:apply-templates select='tech:Class'/>
    </xsl:if> 
    <A name="FUNCTION"/>
    <xsl:if test='tech:Function'>
       <H3>Global Functions</H3>
       <xsl:call-template name="function-summary"/>
       <xsl:call-template name="function-detail"/>
    </xsl:if>
  </xsl:template>



  <xsl:template match="tech:Method">
    <xsl:call-template name="function-or-method"/>
  </xsl:template>

  <xsl:template match="tech:Function" mode="globals">
    <xsl:call-template name="function-or-method"/>
  </xsl:template>

  <xsl:template match="tech:Function">
    <xsl:call-template name="function-or-method"/>
  </xsl:template>

  <xsl:template name="function-or-method">
    <DIV bgcolor='#66FF66'>
      <xsl:call-template name="named-link"/>
      <CODE>
        <xsl:choose>
          <xsl:when test="@init">
            <xsl:text>__init__</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select='tech:Name'/>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>(</xsl:text>
        <xsl:for-each select='tech:Arg'>
          <xsl:value-of select='tech:Name'/>
          <xsl:if test='tech:Default'>
            <xsl:text>=</xsl:text><xsl:value-of select='tech:Default'/>
          </xsl:if>
          <xsl:if test='position() != last()'>, </xsl:if>
        </xsl:for-each>)
      </CODE>
      <BLOCKQUOTE>
        <P><xsl:value-of select='tech:Desc'/></P>

        <B>Parameters</B>
        <xsl:choose>
          <xsl:when test='tech:Arg'>
            <DL>
            <xsl:for-each select='tech:Arg'>
              <DT>
                <CODE>
                  <xsl:choose>
                    <xsl:when test="@init">
                      <I><xsl:text>initializer</xsl:text></I>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:value-of select='tech:Name'/>
                    </xsl:otherwise>
                  </xsl:choose>
                </CODE>
                of type
                <I><xsl:apply-templates select='tech:Type | tech:Reference'/></I><BR/>
              </DT>
              <DD>
                <P><xsl:value-of select='tech:Desc'/></P>
              </DD>
            </xsl:for-each>
            </DL>
          </xsl:when>
          <xsl:otherwise>
            <BLOCKQUOTE>None</BLOCKQUOTE>
          </xsl:otherwise>
        </xsl:choose>

        <B>Return Value</B>
        <xsl:choose>
          <xsl:when test='tech:Return'>
            <DL>
              <DT>
                <I><xsl:apply-templates select='tech:Return/tech:Type | tech:Return/tech:Reference'/></I>
              </DT> 
              <DD>
                <P><xsl:value-of select='tech:Return/tech:Desc'/></P>
              </DD>
            </DL>
          </xsl:when>
          <xsl:otherwise>
            <BLOCKQUOTE>
              None
            </BLOCKQUOTE>
          </xsl:otherwise>
        </xsl:choose>

        <xsl:if test='tech:Exception'>
          <B>Throws</B>
          <xsl:for-each select='tech:Exception'>
            <DL>
              <DT>
                <I><xsl:apply-templates select='tech:Type | tech:Reference'/></I>
              </DT> 
              <DD>
                <P><xsl:value-of select='tech:Desc'/></P>
              </DD>
            </DL>
          </xsl:for-each>
        </xsl:if>            

      </BLOCKQUOTE>
    </DIV>
    <BR/>
    <HR/>
  </xsl:template>



  <!-- Summary functions -->

  <xsl:template name="function-summary">
    <xsl:call-template name='generic-summary'>
      <xsl:with-param name='color' select='"66FF66"'/>
      <xsl:with-param name='type' select='"Global Function"'/>
      <xsl:with-param name='objects' select='tech:Function'/>
    </xsl:call-template>
  </xsl:template>


  <xsl:template name="class-summary">
    <xsl:call-template name='generic-summary'>
      <xsl:with-param name='color' select='"66FF66"'/>
      <xsl:with-param name='type' select='"Class"'/>
      <xsl:with-param name='objects' select='tech:Class'/>
    </xsl:call-template>
  </xsl:template>


  <xsl:template name='method-summary'>
    <xsl:param name='color'/>
    <xsl:call-template name='generic-summary'>
      <xsl:with-param name='color' select='$color'/>
      <xsl:with-param name='type' select='"Method"'/>
      <xsl:with-param name='objects' select='tech:Method'/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name='attribute-summary'>
    <xsl:param name='color'/>
    <xsl:call-template name='generic-summary'>
      <xsl:with-param name='color' select='$color'/>
      <xsl:with-param name='type' select='"Attribute"'/>
      <xsl:with-param name='objects' select='tech:Attribute'/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name='generic-summary'>
    <xsl:param name='color'/>
    <xsl:param name='type'/>
    <xsl:param name='objects'/>
    <xsl:if test="$objects">
      <TABLE BORDER="1" CELLSPACING="0" CELLPADDING="3" WIDTH="600">
        <TR>
          <TD COLSPAN="2" BGCOLOR="#{$color}">
            <BIG><B><xsl:value-of select='$type'/> Summary</B></BIG>
          </TD>
        </TR>
        <xsl:for-each select='$objects'>
          <TR>
            <TD ALIGN="left" WIDTH="150" NOWRAP='NOWRAP'>
              <CODE><xsl:call-template name='goto-named'/></CODE>
            </TD>
            <TD WIDTH="100%"><xsl:value-of select='tech:Desc'/>&#160;</TD>
          </TR>
        </xsl:for-each>
      </TABLE>
      <P>&#160;</P>
    </xsl:if>
  </xsl:template>



  <!-- detail displays -->
  <xsl:template name='function-detail'>
    <xsl:if test='tech:Function'>
      <TABLE BORDER="1" CELLSPACING="0" CELLPADDING="3" WIDTH="600" BGCOLOR="#66FF66">
        <TR><TD>
          <BIG><B>Global Function Details</B></BIG>
        </TD></TR>
      </TABLE>
      <xsl:apply-templates select="tech:Function" mode="globals"/>
    </xsl:if>
  </xsl:template>


  <xsl:template name='method-detail'>
    <xsl:param name='color'/>
    <xsl:if test='tech:Method'>
      <TABLE BORDER="1" CELLSPACING="0" CELLPADDING="3" WIDTH="600" BGCOLOR="#{$color}">
        <TR><TD>
          <BIG><B>Method Details</B></BIG>
        </TD></TR>
      </TABLE>
      <xsl:apply-templates select='tech:Method'/>
    </xsl:if>
  </xsl:template>

  <xsl:template name='member-detail'>
    <xsl:param name='color'/>
    <xsl:if test='tech:Member'>
      <TABLE BORDER="1" CELLSPACING="0" CELLPADDING="3" WIDTH="600" BGCOLOR="#{$color}">
        <TR><TD>
          <BIG><B>Member Details</B></BIG>
        </TD></TR>
      </TABLE>
      <xsl:apply-templates select='tech:Member'/>
    </xsl:if>
  </xsl:template>



  <xsl:template name='attribute-detail'>
    <xsl:param name='color'/>
    <xsl:if test="tech:Attribute">
      <TABLE BORDER="1" CELLSPACING="0" CELLPADDING="3" WIDTH="600" BGCOLOR="#{$color}">
        <TR><TD>
          <BIG><B>Attribute Details</B></BIG>
        </TD></TR>
      </TABLE>

      <BLOCKQUOTE>
      <DL>
        <xsl:for-each select="tech:Attribute">
          <DT>
            <CODE><xsl:call-template name='named-link'/></CODE>
            of type
            <I><xsl:apply-templates select='tech:Type | tech:Reference'/></I><BR/>
          </DT>
          <DD>
           <P><xsl:value-of select='tech:Desc'/></P>
          </DD>
          <BR/>
          <HR/>
        </xsl:for-each>
      </DL>
      </BLOCKQUOTE>
    </xsl:if>
  </xsl:template>


  <xsl:template name='name-desc'>
    <xsl:param name='extra'/>
    <xsl:call-template name="named-link">
      <xsl:with-param name='extra'><xsl:value-of select='$extra'/></xsl:with-param>
    </xsl:call-template>
    <xsl:if test='tech:Desc'>
      <P><xsl:value-of select='tech:Desc'/></P>
    </xsl:if>
  </xsl:template>

  <xsl:template name='named-link'>
    <xsl:param name='extra'/>
    <xsl:param name='fullname'><xsl:call-template name="get-full-name"/></xsl:param>
    <A NAME="{$fullname}"/><H4><xsl:value-of select='concat($extra,tech:Name)'/></H4>
  </xsl:template>

  <xsl:template name='goto-named'>
    <xsl:param name='name'><xsl:value-of select='tech:Name'/></xsl:param>
    <xsl:param name='fullname'><xsl:call-template name="get-full-name"/></xsl:param>
    <A HREF="#{$fullname}"><B><xsl:value-of select='$name'/></B></A>    
  </xsl:template>

<!-- 

<xsl:key name='full-name' match='tech:*/tech:Name' use='ft:join(ancestor::tech:*[../..]/tech:Name, ".")'/>

-->

  <xsl:template name='get-full-name'>
    <xsl:param name="source" select="."/>
    <xsl:value-of select='ft:join($source/ancestor-or-self::tech:*[../..]/tech:Name, ".")'/>
  </xsl:template>

  <xsl:template match='tech:Name'/>
  <xsl:template match="tech:Desc"/>

  <xsl:template match="tech:Reference">
    <xsl:param name="current-name" select="text()"/>
    <xsl:choose>
      <xsl:when test="@document">
        <A href="{@document}#{$current-name}"><xsl:value-of select="text()"/></A>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="name" select="key('Names',$current-name)"/>
        <xsl:variable name="fullname">
          <xsl:call-template name="get-full-name">
            <xsl:with-param name="source" select="$name"/>
          </xsl:call-template>
        </xsl:variable>
        <A href="#{$fullname}"><xsl:value-of select="text()"/></A>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template match='dc:*' priority='-1'/>

</xsl:stylesheet>
