<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:ft="http://namespaces.fourthought.com/common"
  xmlns:c="http://namespaces.fourthought.com/contractual"
  xmlns:doc="http://docbook.org/docbook/xml/4.0/namespace"
  xmlns:fo="http://www.w3.org/1999/XSL/Format"
  version="1.0"
>

  <xsl:import href='sow.common.xslt'/>
  <xsl:import href='html.fo.xslt'/>
  <xsl:import href='docbook.fo.xslt'/>

  <xsl:variable name='header' select='/c:Sow/c:Header'/>

  <xsl:template match="c:Sow">
<fo:root>
  <fo:layout-master-set>
    <fo:simple-page-master  master-name="US-Letter"
      page-height="11in"   page-width="8.5in"
      margin-top="0.5in"   margin-bottom="0.5in"
      margin-left="0.5in"  margin-right="0.5in">
      <fo:region-body/>
    </fo:simple-page-master>
    <fo:simple-page-master  master-name="first"
      page-height="11in"   page-width="8.5in"
      margin-top="0.5in"   margin-bottom="0.5in"
      margin-left="0.5in"  margin-right="0.5in">
      <fo:region-body margin-top='0.5in' margin-bottom='0.5in'/>
      <fo:region-after extent='0.25in'/>
    </fo:simple-page-master>
    <fo:simple-page-master  master-name="main"
      page-height="11in"   page-width="8.5in"
      margin-top="0.5in"   margin-bottom="0.5in"
      margin-left="0.5in"  margin-right="0.5in">
      <fo:region-before extent='1in'/>
      <fo:region-body margin-top='0.5in' margin-bottom='0.5in'/>
      <fo:region-after extent='0.25in'/>
    </fo:simple-page-master>
 
    <fo:page-sequence-master master-name='contents'>
      <fo:repeatable-page-master-alternatives>
        <fo:conditional-page-master-reference
          master-reference='first'
          page-position='first'/>
        <fo:conditional-page-master-reference
          master-reference='main'
          page-position='rest' />
      </fo:repeatable-page-master-alternatives>
    </fo:page-sequence-master>
  </fo:layout-master-set>

  <fo:page-sequence master-reference='contents'>
 
    <fo:flow flow-name="xsl-region-body">
      <fo:block font-size="20pt" font-family="sans-serif"
        font-weight="bold" space-before="6pt" space-after="20pt">
        <xsl:text/>Statement of Work: <xsl:value-of select='@title'/>
      <!--fo:block font-size="16pt" font-family="sans-serif" font-weight="bold" color="green" space-before="6pt" space-after="6pt"-->
      </fo:block>
      <!--fo:block>
      </fo:block-->
      <xsl:apply-templates/>              

      <fo:block space-before="20pt">
The signatures of the authorized representatives of <xsl:value-of select="$provider-abbrev"/> and <xsl:value-of select="$header/c:Client/@name"/>
 shall constitute acceptance
of this Statement of Work as set forth above.
      </fo:block>
      <fo:block space-before="20pt">
        <xsl:text>______________________________________________________________________</xsl:text>
      </fo:block>
      <fo:table>
        <fo:table-column column-width='300pt'/>
        <fo:table-column column-width='100pt'/>
        <fo:table-body>
          <fo:table-row>
            <fo:table-cell>
              <fo:block text-align="start">
                <xsl:value-of select="$header/c:Contractor/c:Contact[@desc='Primary']/c:Name"/>
              </fo:block>
            </fo:table-cell>
            <fo:table-cell text-align="end">
              <fo:block>
                <xsl:text>Date</xsl:text>
              </fo:block>
            </fo:table-cell>
          </fo:table-row>
        </fo:table-body>
      </fo:table>
      <fo:block><xsl:value-of select="$header/c:Contractor/c:Contact[@desc='Primary']/c:Title"/>
      </fo:block>
      <fo:block><xsl:value-of select="$provider-name"/></fo:block>

      <fo:block space-before="20pt">
        <xsl:text>______________________________________________________________________</xsl:text>
      </fo:block>
      <fo:table>
        <fo:table-column column-width='300pt'/>
        <fo:table-column column-width='100pt'/>
        <fo:table-body>
          <fo:table-row>
            <fo:table-cell>
              <fo:block text-align="start">
                <xsl:value-of select="$header/c:Client/c:Contact[@desc='Primary']/c:Name"/>
              </fo:block>
            </fo:table-cell>
            <fo:table-cell text-align="end">
              <fo:block>
                <xsl:text>Date</xsl:text>
              </fo:block>
            </fo:table-cell>
          </fo:table-row>
        </fo:table-body>
      </fo:table>
      <fo:block><xsl:value-of select="$header/c:Client/c:Contact[@desc='Primary']/c:Title"/>
      </fo:block>
      <fo:block><xsl:value-of select="$header/c:Client/@name"/></fo:block>
    </fo:flow>

  </fo:page-sequence>
</fo:root>
 
  </xsl:template>

  <xsl:template match="c:Other">
    <xsl:apply-templates select="doc:*"/>
  </xsl:template>

  <xsl:template match="c:Overview">
    <xsl:call-template name="H3">
      <xsl:with-param name="content">Overview</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates select="doc:*"/>
  </xsl:template>

  <xsl:template match="c:ProjectPlan">
    <xsl:call-template name="H3">
      <xsl:with-param name="content">Project Plan</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates select="doc:*"/>
  </xsl:template>

  <xsl:template match="c:Cost">
    <xsl:call-template name="H3">
      <xsl:with-param name="content">Project Cost</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates select="doc:*"/>
  </xsl:template>

  <xsl:template match="c:ChangeControl">
    <xsl:call-template name="H3">
      <xsl:with-param name="content">Change Control</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates select="doc:*"/>
  </xsl:template>

  <xsl:template match="c:Requirements">
    <xsl:call-template name="H3">
      <xsl:with-param name="content">Project Requirements</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="c:Terminology">
    <xsl:call-template name="H3">
      <xsl:with-param name="content">Terminology</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="c:Header">
    <fo:table font-size="8pt">
      <fo:table-column column-width='150pt'/>
      <fo:table-column column-width='250pt'/>
      <fo:table-body>
        <xsl:apply-templates/>
        <fo:table-row>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:text>Description</xsl:text>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell>
            <fo:block>
              <xsl:value-of select="@desc"/>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>
        <fo:table-row>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:text>Start Date</xsl:text>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:value-of select="@initDate"/>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>
      </fo:table-body>
    </fo:table>
  </xsl:template>

  <xsl:template match="c:Client">
        <fo:table-row>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:text>Client</xsl:text>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:value-of select="@name"/>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>
        <xsl:for-each select="c:Contact">
        <fo:table-row>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:text/>Client's <xsl:value-of select="@desc"/> Contact<xsl:text/>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:value-of select="c:Name"/>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>
        </xsl:for-each>
  </xsl:template>

  <xsl:template match="c:Contractor">
        <fo:table-row>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:text>Contractor</xsl:text>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:value-of select="@name"/>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>
        <xsl:for-each select="c:Contact">
        <fo:table-row>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:text/>Contractor's <xsl:value-of select="@desc"/> Contact<xsl:text/>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:value-of select="c:Name"/>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>
        </xsl:for-each>
  </xsl:template>

  <!--
  <xsl:template match="c:Contact">
        <fo:table-row>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:text/>Contractor's <xsl:value-of select="@desc"/> Contact<xsl:text/>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell>
            <fo:block text-align="start">
              <xsl:value-of select="c:Name"/>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>
  </xsl:template>
-->

</xsl:stylesheet>
