########################################################################
#
# File Name:            TreeCompare.py
#
# Documentation:        http://docs.4suite.org/Lib/TreeCompare.py.html
#
"""
Tools to compare two XML strings.  Mostly for test suites
WWW: http://4suite.org/4DOM         e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""
import re, sgmllib
#import types
from sgmllib import SGMLParser
from xml.dom import Node

# adds colon to regex patterns for names
sgmllib.tagfind = re.compile('[a-zA-Z][-:_.a-zA-Z0-9]*')
sgmllib.attrfind = re.compile(r'\s*([a-zA-Z_][-:_.a-zA-Z0-9]*)(\s*=\s*'
                              r'(\'[^\']*\'|"[^"]*"|\S*))?')

from Ft.Lib.Uri import BASIC_RESOLVER
from Ft.Xml import Domlette, InputSource
from Ft.Xml.Domlette import XmlStrStrip

import HtmlPrinter
FORBIDDEN_END_ELEMENTS = HtmlPrinter.HtmlPrinter.forbiddenEndElements.keys()
del HtmlPrinter

g_htmlTest = re.compile("(<!doctype html)|(<html)",re.IGNORECASE)
g_xmlTest = re.compile(r'<\?xml')

def NoWsTreeCompare(expected,compared):
    return TreeCompare(expected,compared,1)

def TreeCompare(expected, compared, ignoreWhitespace=0, baseUri=None):
    # See if we need to use XML or HTML
    if not g_xmlTest.match(expected) and g_htmlTest.search(expected):
        return not CompareHTML(expected, compared, ignoreWhitespace)

    uri = BASIC_RESOLVER.normalize('expected', baseUri or 'file:')
    try:
        doc1 = Domlette.NonvalidatingReader.parseString(str(expected), uri)
    except:
        print '--- Expected ---'
        print expected
        raise

    uri = BASIC_RESOLVER.normalize('compared', baseUri or 'file:')
    try:
        doc2 = Domlette.NonvalidatingReader.parseString(compared, uri)
    except:
        print '--- Compared ---'
        print compared
        raise

    result = NodeCompare(doc1, doc2, ignoreWhitespace)
    return not result


def NodeCompare(node1, node2, ignoreWhitespace=0):
    if node1.nodeType != node2.nodeType:
        return ReportError(node1, node2, 'nodeType')

    if node1.nodeType == Node.DOCUMENT_NODE:
        for child1, child2 in map(None, node1.childNodes, node2.childNodes):
            if not NodeCompare(child1, child2, ignoreWhitespace):
                return 0

    elif node1.nodeType == Node.DOCUMENT_TYPE_NODE:
        if node1.name != node2.name:
            return ReportError(node1, node2, 'name')
        if node1.publicId != node2.publicId:
            return ReportError(node1, node2, 'publicId')
        if node1.systemId != node2.systemId:
            return ReportError(node1, node2, 'systemId')
        if node1.internalSubset != node2.internalSubset:
            return ReportError(node1, node2, 'internalSubset')
        if len(node1.entities) != len(node2.entities):
            return ReportError(node1, node2, 'entities')
        if len(node1.notations) != len(node2.notations):
            return ReportError(node1, node2, 'notations')

    elif node1.nodeType == Node.ELEMENT_NODE:
        if node1.localName != node2.localName:
            return ReportError(node1, node2, 'localName')
        if node1.namespaceURI != node2.namespaceURI:
            return ReportError(node1, node2, 'namespaceURI')
        attrs1 = node1.attributes.values()
        attrs2 = node2.attributes.values()
        attrs1.sort(lambda a, b: cmp(a.name, b.name))
        attrs2.sort(lambda a, b: cmp(a.name, b.name))
        if len(attrs1) != len(attrs2):
            return ReportError(node1, node2, 'attributes')
        for attr1, attr2 in map(None, attrs1, attrs2):
            if attr1.localName != attr2.localName:
                print node1.attributes.keys()
                print node2.attributes.keys()
                return ReportError(attr1, attr2, 'localName')
            if attr1.namespaceURI != attr2.namespaceURI:
                return ReportError(attr1, attr2, 'namespaceURI')
        if len(node1.childNodes) != len(node2.childNodes):
            return ReportError(node1, node2, 'childNodes')
        for child1, child2 in map(None, node1.childNodes, node2.childNodes):
            if not NodeCompare(child1, child2, ignoreWhitespace):
                return 0

    elif node1.nodeType == Node.TEXT_NODE:
        text1 = node1.data
        text2 = node2.data
        if ignoreWhitespace:
            #FIXME: We can change these to use XmlStrStrip intead
            #(which is technically correct) once all emitters
            #put out Unicode
            #if not XmlStrStrip(text1):
            #    text1 = None
            #if not XmlStrStrip(text2):
            #    text2 = None
            #import string
            if not text1.strip():
                text1 = None
            if not text2.strip():
                text2 = None
        if cmp(text1, text2):
            return ReportError(node1, node2, 'data')

    elif node1.nodeType == Node.COMMENT_NODE:
        if node1.data != node2.data:
            return ReportError(node1, node2, 'data')

    elif node1.nodeType == Node.PROCESSING_INSTRUCTION_NODE:
        if node1.target != node2.target:
            return ReportError(node1, node2, 'target')
        if node1.data != node2.data:
            return ReportError(node1, node2, 'data')

    # All tests pass, they are the same
    return 1


def ReportError(node1, node2, attribute):
    import pprint
    print '--- expected ---'
    PrintParentage(node1)
    print 'node: %s' % repr(node1)
    print 'node.%s:' % attribute
    pprint.pprint(getattr(node1, attribute))
    print '--- compared ---'
    PrintParentage(node2)
    print 'node: %s' % repr(node2)
    print 'node.%s:' % attribute
    pprint.pprint(getattr(node2, attribute))
    return 0

def PrintParentage(node):
    nodes = [node]
    if node.nodeType == Node.ATTRIBUTE_NODE:
        parent = node.ownerElement
    else:
        parent = node.parentNode
    while parent:
        nodes.insert(0, parent)
        parent = parent.parentNode
    indent = ''
    for node in nodes:
        print '%s%s' % (indent, node.nodeName)
        indent = indent + '  '



class SGMLParserEventGenerator(SGMLParser):

    TEXT_EVENT = 1
    COMMENT_EVENT = 2
    START_TAG_EVENT = 3
    END_TAG_EVENT = 4
    ENTITYREF_EVENT = 6
    CHARREF_EVENT = 7

    def __init__(self, verbose=0):
        self.testdata = ""
        SGMLParser.__init__(self, verbose)
        self.events = []
        self.ignoreable_ws = 1

    def handle_data(self, data):
        self.testdata = self.testdata + data

    def flush(self):
        data = self.testdata
        if data:
            self.testdata = ""
            if not self.ignoreable_ws:
                self.events.append((self.TEXT_EVENT,data))

    def handle_comment(self, data):
        self.flush()
        self.events.append((self.COMMENT_EVENT,data))

    def unknown_starttag(self, tagname, attrs):
        self.flush()
        if self.ignoreable_ws:
            self.ignoreable_ws = 0

        # Convert list of tuples to dictionary
        dict = {}
        for name, value in attrs:
            dict[name] = value
        self.events.append((self.START_TAG_EVENT, tagname, dict))
        if tagname.lower() in FORBIDDEN_END_ELEMENTS:
            self.events.append((self.END_TAG_EVENT,tagname))
        return

    def unknown_endtag(self, tag):
        self.flush()
        self.events.append((self.END_TAG_EVENT,tag))

    def unknown_entityref(self, ref):
        self.flush()
        self.events.append((self.ENTITYREF_EVENT,ref))

    def unknown_charref(self, ref):
        self.flush()
        self.events.append((self.CHARREF_EVENT,ref))

    def close(self):
        SGMLParser.close(self)
        self.flush()

g_xmlEmptyTagPattern = re.compile(
    '<([a-zA-Z][-:_.a-zA-Z0-9]*\s*([a-zA-Z_][-:_.a-zA-Z0-9]*)(\s*=\s*'
    r'(\'[^\']*\'|"[^"]*"|\S*))?)/>')

def CompareHTML(html1,html2,ignoreWhitespace):
    # SGMLParser goes awry when <emptyelement/> tags are encountered.
    # Obviously this is a very kludgy 'solution'
    html1 = g_xmlEmptyTagPattern.sub(r'<\1>',html1)
    html2 = g_xmlEmptyTagPattern.sub(r'<\1>',html2)

    p1 = SGMLParserEventGenerator()
    p1.feed(html1)
    p1.close()

    p2 = SGMLParserEventGenerator()
    p2.feed(html2)
    p2.close()

    stack = []  #Not 100% accurate, but close enough
    for cur1, cur2 in zip(p1.events, p2.events):
        if cur1[0] != cur2[0]:
            #different events
            return ReportEventError(cur1,cur2,stack,'different events')
        event = cur1[0]
        if event == SGMLParserEventGenerator.TEXT_EVENT:
            # Compare the text of each
            d1 = cur1[1]
            d2 = cur2[1]
            if d1.strip() != d2.strip():
                return ReportEventError(cur1,cur2,stack,'data')
        elif event == SGMLParserEventGenerator.COMMENT_EVENT:
            d1 = cur1[1]
            d2 = cur2[1]
            if d1.strip() != d2.strip():
                return ReportEventError(cur1,cur2,stack,'comment data')
        elif event == SGMLParserEventGenerator.START_TAG_EVENT:
            if cur1[1] != cur2[1]:
                return ReportEventError(cur1,cur2,stack,'start tag name')
            stack.append(cur1[1])  #Save for a nice print out

            att1 = cur1[2]
            att2 = cur2[2]

            if len(att1) != len(att2):
                return ReportEventError(cur1,cur2,stack,'number of attributes')

            for name, value in att1.items():
                if att2.get(name, -1) != value:
                    return ReportEventError(cur1,cur2,stack,'attribute value %s' % name)
        elif event == SGMLParserEventGenerator.END_TAG_EVENT:
            if cur1[1] != cur2[1]:
                return ReportEventError(cur1,cur2,stack,'end tag name')
            while stack and stack[-1] != cur1[1]:  #Remove it
                del stack[-1]
            del stack[-1]
        elif event == SGMLParserEventGenerator.ENTITYREF_EVENT:
            if cur1[1] != cur2[1]:
                return ReportEventError(cur1,cur2,stack,'entity ref')
        elif event == SGMLParserEventGenerator.CHARREF_EVENT:
            if cur1[1] != cur2[1]:
                return ReportEventError(cur1,cur2,stack,'char ref')
        else:
            raise cur1
    return 1

def ReportEventError(event1, event2, stack, attribute):
    PrintStack(stack)
    print '--- Expected ---'
    print attribute, repr(event1[1:])
    print '--- Compared ---'
    print attribute, repr(event2[1:])
    return 0

def PrintStack(stack):
    indent = ''
    for name in stack:
        print "%s%s" % (indent,name)
        indent += '  '


##        if isHtml:
##            # HTML DOM should already capitalize all tagNames
##            if node1.tagName != node2.tagName:
##                return ReportError(node1, node2, 'tagName')
##            # Elements where whitespace is significant
##            if node1.tagName in ['SCRIPT', 'STYLE', 'PRE', 'TEXTAREA']:
##                ignoreWhitespace = 0


##            if isHtml:
##                # HTML DOMs should force upper case already
##                # FIXME: PyXML 0.7 changed HTML attributes to be NS so they
##                # are no longer forced to uppercase!
##                if attr1.name.upper() != attr2.name.upper():
##                    return ReportError(attr1, attr2, 'name')


##        if ignoreWhitespace or isHtml:
##            #FIXME: We can change these to use XmlStrStrip intead
##            #(which is technically correct) once all emitters
##            #put out Unicode
##            #if not XmlStrStrip(text1):
##            #    text1 = None
##            #if not XmlStrStrip(text2):
##            #    text2 = None
##            #import string
##            if not text1.strip():
##                text1 = None
##            if not text2.strip():
##                text2 = None
